<?php

namespace Bbg\DotDatConverter;

use Bbg\DotDatConverter\Entities\Schema;
use Bbg\DotDatConverter\Exceptions\FileNotFoundException;
use Bbg\DotDatConverter\Helpers\DataParser;
use Bbg\DotDatConverter\Helpers\DataValidator;
use Bbg\DotDatConverter\Helpers\SchemaLoader;
use Bbg\DotDatConverter\Helpers\SchemaValidator;

class DotDatConverter
{
    private Schema $schema;
    private string $endOfLine = PHP_EOL;
    private bool $validateSchema;
    private bool $validateData;

    public function __construct(string $schemaFile, bool $validateSchema = true, bool $validateData = true)
    {
        $this->validateSchema = $validateSchema;
        $this->validateData = $validateData;
        $this->schema = SchemaLoader::load($schemaFile);

        if ($this->validateSchema) {
            SchemaValidator::validate($this->schema);
        }
    }

    public function encode(array $data): string
    {
        return '';
    }

    public function recordEncode(array $data): string
    {
        if ($this->validateData) {
            $valid = DataValidator::validate($this->schema, $data);
        }

        $lines = $this->recordEncodeLevel($data);

        return implode($this->getEndOfLine(), $lines);
    }

    public function decode(string $data): array
    {
        return [];
    }

    public function recordDecode(string $data): array
    {
        return DataParser::parse($this->schema, $data);
    }


    public function fileDecode(string $datFile): array
    {
     $data = [];

        if (!file_exists($datFile) || !is_readable($datFile)) {
            throw new FileNotFoundException('File "' . $datFile . '" not found');
        }

        try {
            $fh = fopen($datFile, 'r');

            while (!feof($fh)) {
                $data[] = $this->recordDecode(fgets($fh));
            }

            fclose($fh);
        } catch (\Exception $exception) {

        }

        return $data;
    }

    /**
     * @return string
     */
    public function getEndOfLine(): string
    {
        return $this->endOfLine;
    }

    /**
     * @param string $endOfLine
     */
    public function setEndOfLine(string $endOfLine): self
    {
        $this->endOfLine = $endOfLine;
        return $this;
    }

    private function recordEncodeLevel(array $data, array $lines = []): array
    {
        foreach ($data as $section => $values) {
            if (!is_array($values)) {
                continue;
            }

            $section = $this->schema->getSection($section);

            if (is_null($section)) {
                continue;
            }

            if (!$section->isRepeatableInRecord() || DataValidator::getArrayDepth($values) < 1) {
                $section->setValues($values);
                $lines[] = $section->toString();
                continue;
            }

            foreach ($values as $key => $items) {
                if (is_int($key)) {
                    $section->setValues($items);
                    $lines[] = $section->toString();
                } elseif ($key == 'childs') {
                    $lines = $this->recordEncodeLevel($items, $lines);
                }
            }
        }

        return $lines;
    }
}