<?php

namespace Bbg\DotDatConverter\Entities;

use Bbg\DotDatConverter\Exceptions\InvalidJsonException;
use Bbg\DotDatConverter\Exceptions\UnprocessableDataException;

class Schema implements \Iterator
{
    private array $sections = [];
    private int $position = 0;
    private string $checksum;

    public function __construct(object $data)
    {
        if (!isset($data->sections)) {
            throw new UnprocessableDataException('No sections found in data object');
        }

        $this->checksum = sha1(serialize($data));

        foreach ($data->sections as $section) {
            $sectionObject  = new Section($section);
            $this->sections[$sectionObject->getName()] = $sectionObject;
        }
    }

    /**
     * @return string
     */
    public function getChecksum(): string
    {
        return $this->checksum;
    }

    /**
     * @return array
     */
    public function getSections(): array
    {
        return $this->sections;
    }

    public function getSection(string $name, ?array $values = null): ?Section
    {
        /* @var \Bbg\DotDatConverter\Entities\Section $section */
        $section = ($this->sections[$name] ?? null);

        if ($section instanceof Section && is_array($values)){
            $section->setValues($values);
        }

        return $section;
    }

    public function getSectionByIdentifier(string $identifier): ?Section
    {
        foreach ($this->getSections() as $section){
            /* @var \Bbg\DotDatConverter\Entities\Section $section */

            if($section->getIdentifier() == $identifier){
                return $section;
            }
        }

        return null;
    }

    public function current(): mixed
    {
        return $this->sections[$this->position];
    }

    public function next(): void
    {
        ++$this->position;
    }

    public function key(): mixed
    {
        return $this->position;
    }

    public function valid(): bool
    {
        return isset($this->sections[$this->position]);
    }

    public function rewind(): void
    {
        $this->position = 0;
    }
}