<?php

namespace Bbg\DotDatConverter\Entities;

use Bbg\DotDatConverter\Exceptions\UnprocessableDataException;

class Section implements \Iterator
{
    private array $fields = [];
    private string $name;
    private bool $repeatableInRecord = false;
    private bool $repeatableInFile = false;
    private bool $required = false;
    private bool $startNewRecord = false;
    private int $position = 0;
    private string $identifier;

    public function __construct(object $data)
    {
        if (isset($data->name)) {
            $this->setName($data->name);
        }

        if (isset($data->repeatableInRecord)) {
            $this->setRepeatableInRecord($data->repeatableInRecord);
        }

        if (isset($data->repeatableInFile)) {
            $this->setRepeatableInFile($data->repeatableInFile);
        }

        if (isset($data->required)) {
            $this->setRequired($data->required);
        }

        if (isset($data->startNewRecord)) {
            $this->setStartNewRecord($data->required);
        }

        if (!isset($data->fields)) {
            throw new UnprocessableDataException('No fields found in data object');
        }

        foreach ($data->fields as $index => $field) {
            $fieldObject = new Field($field);

            if ($index === 0) {
                $this->setIdentifier($fieldObject->getDefault());
            }

            $this->fields[$fieldObject->getName()] = $fieldObject;
        }
    }

    public function __toString(): string
    {
        return $this->toString();
    }

    public function toString(): string
    {
        $line = '';

        foreach ($this->fields as $field) {
            $line .= (string)$field;
        }

        return $line;
    }

    public function setValues(array $values): self
    {
        foreach ($values as $field => $value) {
            if (isset($this->fields[$field])) {
                $this->fields[$field]->setValue($value);
            }
        }
        return $this;
    }

    /**
     * @return array
     */
    public function getFields(): array
    {
        return $this->fields;
    }

    /**
     * @return string
     */
    public function getIdentifier(): string
    {
        return $this->identifier;
    }

    /**
     * @param string $identifier
     */
    public function setIdentifier(string $identifier): self
    {
        $this->identifier = $identifier;
        return $this;
    }

    /**
     * @return string
     */
    public function getName(): string
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName(string $name): self
    {
        $this->name = $name;
        return $this;
    }

    /**
     * @return bool
     */
    public function isRepeatableInFile(): bool
    {
        return $this->repeatableInFile;
    }

    /**
     * @return bool
     */
    public function isRepeatableInRecord(): bool
    {
        return $this->repeatableInRecord;
    }

    /**
     * @return bool
     */
    public function isRequired(): bool
    {
        return $this->required;
    }

    /**
     * @return bool
     */
    public function isStartNewRecord(): bool
    {
        return $this->startNewRecord;
    }

    /**
     * @param bool $repeatableInFile
     */
    public function setRepeatableInFile(bool $repeatableInFile): self
    {
        $this->repeatableInFile = $repeatableInFile;
        return $this;
    }

    /**
     * @param bool $repeatableInRecord
     */
    public function setRepeatableInRecord(bool $repeatableInRecord): self
    {
        $this->repeatableInRecord = $repeatableInRecord;
        return $this;
    }

    /**
     * @param bool $required
     */
    public function setRequired(bool $required): self
    {
        $this->required = $required;
        return $this;
    }

    /**
     * @param bool $startNewRecord
     */
    public function setStartNewRecord(bool $startNewRecord): void
    {
        $this->startNewRecord = $startNewRecord;
    }

    public function current(): mixed
    {
        return $this->fields[$this->position];
    }

    public function next(): void
    {
        ++$this->position;
    }

    public function key(): mixed
    {
        return $this->position;
    }

    public function valid(): bool
    {
        return isset($this->fields[$this->position]);
    }

    public function rewind(): void
    {
        $this->position = 0;
    }
}