<?php

namespace Bbg\DotDatConverter\Helpers;

use Bbg\DotDatConverter\Entities\Schema;

class DataParser
{
    private static array $recordTypeLengthCache = [];

    public static function parse(Schema $schema, string $data): array
    {
        $length = self::determineRecordTypeLength($schema);
        $type = trim(substr($data, 0, $length));
        $section = $schema->getSectionByIdentifier($type);
        $fields = [];

        if(is_null($section)){
            return [];
        }

        foreach ($section->getFields() as $field){
            /* @var \Bbg\DotDatConverter\Entities\Field $field */

            $fields[$field->getName()] = trim(substr($data, ($field->getStart() - 1), $field->getLength()));
        }

        return $fields;
    }

    public static function determineRecordType(string $schema): string
    {
        return '';
    }

    public static function determineRecordTypeLength(Schema $schema): int
    {
        if(isset(self::$recordTypeLengthCache[$schema->getChecksum()])){
            return self::$recordTypeLengthCache[$schema->getChecksum()];
        }

        $length = [];

        foreach ($schema->getSections() as $section) {
            /* @var \Bbg\DotDatConverter\Entities\Section $section */


            foreach ($section->getFields() as $field) {
                /* @var \Bbg\DotDatConverter\Entities\Field $field */
                $length[] = $field->getLength();
                break;
            }
        }

        $length = self::$recordTypeLengthCache[$schema->getChecksum()] = max($length);

        return $length;
    }
}