<?php

namespace Bbg\DotDatConverter\Helpers;

use Bbg\DotDatConverter\Entities\Schema;
use Bbg\DotDatConverter\Exceptions\EmptySchemaException;
use Bbg\DotDatConverter\Exceptions\FileNotFoundException;
use Bbg\DotDatConverter\Exceptions\InvalidJsonException;

class SchemaLoader
{
    public static function load(string $schemaFile): Schema
    {
        if (!file_exists($schemaFile)) {
            throw new FileNotFoundException('File "' . $schemaFile . '" not found');
        }

        $content = file_get_contents($schemaFile);

        if(empty($content)){
            throw new EmptySchemaException('File "' . $schemaFile . '" is empty');
        }

        $json = json_decode($content);

        if(is_null($json)){
            throw new InvalidJsonException('File is not a valid json file');
        }

        return new Schema($json);
    }
}